package kr.co.mkblog.mkblogopenglesexample;

import android.app.Fragment;
import android.content.Context;
import android.content.res.AssetManager;
import android.opengl.GLES30;
import android.util.Log;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;

public class mkShader {
    public int mkProgramID;

    public mkShader(Context inputContext, String vertexName, String fragmentName) {
        String vertexCode = readCodeFromFile(inputContext, vertexName);
        String fragmentCode = readCodeFromFile(inputContext, fragmentName);
        //Log.d("(MK) mkShader File:", "(Shader Code)" + vertexCode);
        // Log.d("(MK) mkShaderFile:", "(Fragment Code)" + fragmentCode);

        //MK: Vertex Compile
        int mkVertexShader = compileShader(GLES30.GL_VERTEX_SHADER, vertexCode);

        //MK: Fragment Compile
        int mkFragShader = compileShader(GLES30.GL_FRAGMENT_SHADER, fragmentCode);

        mkProgramID = GLES30.glCreateProgram();
        GLES30.glAttachShader(mkProgramID, mkVertexShader);
        GLES30.glAttachShader(mkProgramID, mkFragShader);
        GLES30.glLinkProgram(mkProgramID);

        int[] mkLinkedStatus = new int[1];
        GLES30.glGetProgramiv(mkProgramID, GLES30.GL_LINK_STATUS, mkLinkedStatus, 0);
        if(mkLinkedStatus[0] == 0){
            Log.d("(MK) mkShader File:", GLES30.glGetProgramInfoLog(mkProgramID));
            Log.d("(MK) mkShader File:", "(Error) Failed Linking");
            GLES30.glDeleteProgram(mkProgramID);
        }
        else{
            Log.d("(MK) mkShader File:", "Linked Vertex & Fragment Code");
        }

        GLES30.glDeleteShader(mkVertexShader);
        GLES30.glDeleteShader(mkFragShader);

    }

    public void use(){
        GLES30.glUseProgram(mkProgramID);
    }

    //MK: Uniform Float 값을 설정하기 위해 추가
    public void setUniformFloat(String uniformName, float inputValue){
        int uniformLocation = GLES30.glGetUniformLocation(this.mkProgramID, uniformName);
        GLES30.glUniform1f(uniformLocation, inputValue);
    }

    //MK: Uniform Int 값을 설정하기 위해 추가 // 보통 Texture에서 많이 사용됨
    public void setUniformInt(String uniformName, int inputValue){
        int uniformLocation = GLES30.glGetUniformLocation(this.mkProgramID, uniformName);
        GLES30.glUniform1i(uniformLocation, inputValue);
    }

    public void setUniformMatrix4(String uniformName, float[] matrix){
        int uniformLocation = GLES30.glGetUniformLocation(this.mkProgramID, uniformName);
        GLES30.glUniformMatrix4fv(uniformLocation, 1, false, matrix, 0);
    }


    private String readCodeFromFile(Context inputContext, String fileName){
        BufferedReader reader = null;
        String mkCode = "";
        try {
            InputStreamReader in = new InputStreamReader(inputContext.getAssets().open(fileName));
            reader = new BufferedReader(in);
            // read file until EOF
            String line;
            while ((line = reader.readLine()) != null) {
                mkCode += line;
                mkCode += "\n";
            }
        } catch (IOException e) {
            Log.d("(MK) mkShader File:", "Cannot read file");
        } finally {
            if (reader != null) {
                try {
                    reader.close();
                } catch (IOException e) {
                    Log.d("(MK) mkShader File:", "Cannot close file");
                }
            }
        }
        return mkCode;
    }

    private static int compileShader(int type, String ShaderCode){
        int mkShaderId = GLES30.glCreateShader(type);
        GLES30.glShaderSource(mkShaderId, ShaderCode);
        GLES30.glCompileShader(mkShaderId);
        int[] mkCompileStatue = new int[1];
        GLES30.glGetShaderiv(mkShaderId, GLES30.GL_COMPILE_STATUS, mkCompileStatue, 0);
        if(mkCompileStatue[0] == 0){
            Log.d("(MK) mkShader File:", GLES30.glGetShaderInfoLog(mkShaderId));
            Log.d("(MK) mkShader File:", "(Code) " + ShaderCode + "\n Compiled Error");
            mkShaderId = 0;
        }
        else{
            Log.d("(MK) mkShader File:", "(Code) " + ShaderCode + "\nCompiled");
        }
        return mkShaderId;
    }
}